--if not IsInGuild() then
--  return
--end

local AceGUI = LibStub("AceGUI-3.0")

local dateFormat = "%m/%d/%y %H:%M:%S"
local difficultyNames = {"(10) Normal", "(25) Normal", "(10) Heroic", "(25) Heroic", "(25) Raid Finder"}
difficultyNames[0] = ""
local desires = {"Need", "Minor", "Off-spec"}
local tab
local scrollFrame
local instanceDropdown
local encounterDropdown
local curInstance, curDifficulty, curEncounter
local visible

BossListFrame = CreateFrame("Frame")

local function TimeDifference(lastUpdate)
  local pastDate = date("*t", lastUpdate)
  local curDate = date("*t")
  local yearDif = curDate.year - pastDate.year
  if yearDif > 0 then
    return string.format("%d years ago", yearDif)
  end
  
  local monthDif = curDate.month - pastDate.month
  if monthDif > 0 then
    return string.format("%d months ago", monthDif)
  end
  
  local dayDif = curDate.day - pastDate.day
  if dayDif > 0 then
    return string.format("%d days ago", dayDif)
  end
  
  local hourDif = curDate.hour - pastDate.hour
  if hourDif > 0 then
    return string.format("%d hours ago", hourDif)
  end
  
  local minuteDif = curDate.min - pastDate.min
  if minuteDif > 0 then
    return string.format("%d minutes ago", minuteDif)
  end
  
  local secondDif = curDate.sec - pastDate.sec
  return string.format("%d seconds ago", secondDif)
end

local function SortEntries(a, b)
  return a.desirability < b.desirability
end

local function DrawEntries(container, selectedInstance, selectedDifficulty, selectedEncounter)
  container:ReleaseChildren()
  for instance, difficulties in pairs(BossList_entries) do -- Instances
    if selectedInstance == nil or instance == selectedInstance then -- Filter on instance
      for difficulty, encounters in pairs(difficulties) do -- Difficulties
         if (selectedDifficulty == nil or difficulty == selectedDifficulty) and 
            (selectedEncounter == nil or encounters[selectedEncounter] ~= nil) then 
          -- Filter on difficulty, and check whether or not the selected encounter is in this instance
          
          local isHeaderPrinted = false
          local bossIndex = 1;
          local encounterName, _, encounter, _, _ = EJ_GetEncounterInfoByIndex(bossIndex, instance)
          while encounter ~= nil do           -- Encounters
            characters = encounters[encounter]
            -- Check if it exists and filter
            if characters ~= nil and (selectedEncounter == nil or encounter == selectedEncounter) then 
          
              local isEncounterPrinted = false
              
              sort(characters, SortEntries)
              for character, desire in pairs(characters) do -- Characters
                if desire.desirability < 4 then
                  if not isHeaderPrinted then
                    local instanceName = EJ_GetInstanceInfo(instance)
                    local difficultyName = difficultyNames[difficulty]
                    
                    local instanceHeading = AceGUI:Create("Heading")
                    instanceHeading:SetFullWidth(true)
                    instanceHeading:SetText(instanceName .. " - " .. difficultyName)
                    container:AddChild(instanceHeading)
                    
                    isHeaderPrinted = true
                  end
                  if not isEncounterPrinted then
                    --local encounterName = EJ_GetEncounterInfo(encounter)    
                    local encounterLabel = AceGUI:Create("Label")        
                    encounterLabel:SetColor(1,0.82,0)
                    encounterLabel:SetFullWidth(true)
                    encounterLabel.label:SetJustifyH("LEFT")
                    encounterLabel:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE" )
                    encounterLabel:SetText(encounterName)
                    container:AddChild(encounterLabel)
                    
                    isEncounterPrinted = true
                  end
                  
              
                  local characterLabel = AceGUI:Create("Label")
                  characterLabel:SetRelativeWidth(0.4)
                  characterLabel:SetText(character)
                  characterLabel.label:SetJustifyH("LEFT")
                  container:AddChild(characterLabel)
                  
                  local lastUpdateLabel = AceGUI:Create("Label")
                  lastUpdateLabel:SetRelativeWidth(0.3)
                  lastUpdateLabel:SetText(TimeDifference(desire.lastUpdate))
                  lastUpdateLabel:SetPoint("RIGHT", characterLabel.frame, 0,0)
                  lastUpdateLabel.label:SetJustifyH("CENTER")
                  container:AddChild(lastUpdateLabel)
                  
                  local desireLabel = AceGUI:Create("Label")
                  desireLabel:SetRelativeWidth(0.29)
                  desireLabel:SetText(desires[desire.desirability])
                  desireLabel:SetPoint("RIGHT", lastUpdateLabel.frame, 0,0)
                  --desireLabel:SetImage("Interface/BUTTONS/UI-GroupLoot-Coin-Up")
                  desireLabel:SetImageSize(30,30)
                  desireLabel.label:SetJustifyH("RIGHT")
                  container:AddChild(desireLabel)
                end
              end
              
              if isEncounterPrinted then
                local heading = AceGUI:Create("Heading")
                heading:SetFullWidth(true)
                container:AddChild(heading)
              end
            end
            bossIndex = bossIndex + 1;
            encounterName, _, encounter, _, _ = EJ_GetEncounterInfoByIndex(bossIndex, instance)
          end
        end
      end
    end
  end
end

local function UpdateInstanceList()
  local instances = {}
  for instance, _ in pairs(BossList_entries) do
    instances[instance] = EJ_GetInstanceInfo(instance)
  end
  instances[0] = ""
  BossListInstance = instances

  instanceDropdown:SetList(instances)
end

local function UpdateEncounterList(instance)
  local encounterList = {}
  if instance == 0 or instance == nil then
    curInstance = nil
    
    for _, difficulties in pairs(BossList_entries) do
      for _, encounters in pairs(difficulties) do
        for encounter, _ in pairs(encounters) do
          if encounterList[encounter] == nil then
            encounterList[encounter] = EJ_GetEncounterInfo(encounter)
          end
        end
      end
    end
  elseif BossList_entries[instance] ~= nil then
    curInstance = instance
    
    for _, encounters in pairs(BossList_entries[instance]) do
      for encounter, _ in pairs(encounters) do -- Encounters
        if encounterList[encounter] == nil then
          encounterList[encounter] = EJ_GetEncounterInfo(encounter)
        end
      end
    end
  end
  encounterList[0] = ""
  encounterDropdown:SetList(encounterList)
  encounterDropdown:SetValue(curEncounter)
end


local function FilterInstance(self, event, instance)
  
  curEncounter = nil
  
  UpdateEncounterList(instance)
  DrawEntries(scrollFrame, curInstance, curDifficulty, curEncounter)

end

local function FilterDifficulty(self, event, difficulty)
  if difficulty == 0 then
    curDifficulty = nil
  else
    curDifficulty = difficulty
  end
  DrawEntries(scrollFrame, curInstance, curDifficulty, curEncounter)
end

local function FilterEncounter(self, event, encounter)
  if encounter == 0 then
    curEncounter = nil
  else
    curEncounter = encounter
  end
  DrawEntries(scrollFrame, curInstance, curDifficulty, curEncounter) 
end

local function DrawBossesTab(container)
  local difficultyDropdown = AceGUI:Create("Dropdown")
  difficultyDropdown:SetList(difficultyNames)
  
  difficultyDropdown:SetLabel("Difficulty")
  difficultyDropdown:SetRelativeWidth(0.2)
  difficultyDropdown:SetCallback("OnValueChanged", FilterDifficulty)
  
  encounterDropdown = AceGUI:Create("Dropdown")
  encounterDropdown:SetLabel("Encounter")
  encounterDropdown:SetRelativeWidth(0.4)
  encounterDropdown:SetCallback("OnValueChanged", FilterEncounter)
    
  instanceDropdown = AceGUI:Create("Dropdown")
  UpdateInstanceList()
  instanceDropdown:SetLabel("Instance")
  instanceDropdown:SetRelativeWidth(0.39)
  instanceDropdown:SetCallback("OnValueChanged", FilterInstance)
  
  difficultyDropdown:SetPoint("RIGHT", instanceDropdown.frame, 0, 0)
  encounterDropdown:SetPoint("RIGHT", difficultyDropdown.frame, 0, 0)
  
  container:AddChild(instanceDropdown)
  container:AddChild(difficultyDropdown)
  container:AddChild(encounterDropdown)
  
 
  local scrollContainer = AceGUI:Create("SimpleGroup")
  scrollContainer:SetFullWidth(true)
  scrollContainer:SetFullHeight(true)
  scrollContainer:SetLayout("Fill")
  container:AddChild(scrollContainer)
  
  scrollFrame = AceGUI:Create("ScrollFrame")
  scrollFrame:SetLayout("Flow")
  scrollContainer:AddChild(scrollFrame)
  
  --instanceDropdown:SetValue(0)
  FilterInstance(encounterDropdown, nil, 0)
  
end

local function DrawPlayerTab(container)

end

local function SelectGroup(container, event, group)
  container:ReleaseChildren()
  if group =="tab1" then
    DrawBossesTab(container)
  elseif group == "tab2" then
    DrawPlayerTab(container)
  end
end

BossListFrame.init = function()
  local frame = AceGUI:Create("Frame")
  frame:SetTitle("BossList")
  frame:SetWidth(650)
  frame:SetCallback("OnClose", function(widget) AceGUI:Release(widget); tab = nil; visible = false end)
  frame:SetLayout("Fill")
  
  tab = AceGUI:Create("TabGroup")
  tab:SetLayout("Flow")
  tab:SetTabs({{text="Bosses", value="tab1"}, {text="Players", value="tab2"}})
  tab:SetCallback("OnGroupSelected", SelectGroup)
  tab:SelectTab("tab1")
  frame:AddChild(tab)
  
end


BossListFrame.addEntry = function(instanceID, difficulty, encounterID, characterName, desirability, lastUpdate)
  if BossList_entries[instanceID] == nil then BossList_entries[instanceID] = {} end
  local instanceTable = BossList_entries[instanceID]
  if instanceTable[difficulty] == nil then instanceTable[difficulty] = {} end
  local difficultyTable = instanceTable[difficulty]
  if difficultyTable[encounterID] == nil then difficultyTable[encounterID] = {} end
  local encounterTable = difficultyTable[encounterID]
  

  
  --if desirability == 4 then -- Not needed, remove entry
  --  encounterTable[characterName] = nil
  --  if next(encounterTable) == nil then -- Encounter table is empty, remove it
  --    difficultyTable[encounterID] = nil
  --    
  --    if next(difficultyTable) == nil then -- Difficulty table is empty, remove it
  --      instanceTable[difficulty] =  nil
  --      
  --      if next(instanceTable) ==  nil then -- Instance table is empty, remove it
  --        BossList_entries[instanceID] = nil
  --      end
  --    end
  --  end    
  --else
  lastUpdate = lastUpdate or time()
  
  if time() - lastUpdate < 7776000 then -- Newer than 3 months
    encounterTable[characterName] = {desirability = desirability, lastUpdate}
  end
  
  
  if visible then
    BossListFrame.Show()
  end
end

BossListFrame.Show = function()
  visible = true
  if tab == nil then
    BossListFrame.init()
  else
    UpdateInstanceList()
    UpdateEncounterList(curInstance)
    DrawEntries(scrollFrame, curInstance, curDifficulty, curEncounter)
    collectgarbage()
  end
end

--print("Overview loaded")